@@ -16,25 +16,23 @@ When adding an item or a property, it needs to be kept in sync with different pl
  - It needs to be changed in this module
  - In the compiler: builtins.60
  - In the interpreter (new item only): dynamic_component.rs
- - For the C++ code (new item only):
-   - the cbindgen.rs to export the new item
-   - the `using` declaration in sixtyfps.h for the item and its vtable
+ - For the C++ code (new item only): the cbindgen.rs to export the new item
  - Don't forget to update the documentation
 */
 
 #![allow(unsafe_code)]
 #![allow(non_upper_case_globals)]
-#![allow(missing_docs)] // because documenting each property of items is redundent
+#![allow(missing_docs)] // because documenting each property of items is redundant
 
 use crate::component::ComponentVTable;
 use crate::graphics::PathDataIterator;
-use crate::graphics::{Brush, Color, PathData, Rect, Size};
+use crate::graphics::{Brush, Color, PathData, Rect};
 use crate::input::{
     FocusEvent, InputEventFilterResult, InputEventResult, KeyEvent, KeyEventResult, KeyEventType,
-    MouseEvent, MouseEventType,
+    MouseEvent,
 };
 use crate::item_rendering::CachedRenderingData;
-use crate::layout::LayoutInfo;
+use crate::layout::{LayoutInfo, Orientation};
 #[cfg(feature = "rtti")]
 use crate::rtti::*;
 use crate::window::ComponentWindow;
@@ -49,7 +47,7 @@ pub use text::*;
 mod image;
 pub use self::image::*;
 
-/// Alias for `&mut dyn ItemRenderer`. Required so cbingen generates the ItemVTable
+/// Alias for `&mut dyn ItemRenderer`. Required so cbindgen generates the ItemVTable
 /// despite the presence of trait object
 type ItemRendererRef<'a> = &'a mut dyn crate::item_rendering::ItemRenderer;
 
@@ -57,6 +55,22 @@ type ItemRendererRef<'a> = &'a mut dyn crate::item_rendering::ItemRenderer;
 pub type VoidArg = ();
 type KeyEventArg = (KeyEvent,);
 
+#[macro_export]
+macro_rules! declare_item_vtable {
+    (fn $getter:ident() -> $item_vtable_ty:ident for $item_ty:ty) => {
+        ItemVTable_static! {
+            #[no_mangle]
+            pub static $item_vtable_ty for $item_ty
+        }
+        #[no_mangle]
+        #[cfg(all(feature = "ffi", windows))]
+        pub extern "C" fn $getter() -> *const ItemVTable {
+            use vtable::HasStaticVTable;
+            <$item_ty>::static_vtable()
+        }
+    };
+}
+
 /// Items are the nodes in the render tree.
 #[vtable]
 #[repr(C)]
@@ -69,22 +83,22 @@ pub struct ItemVTable {
     /// Returns the geometry of this item (relative to its parent item)
     pub geometry: extern "C" fn(core::pin::Pin<VRef<ItemVTable>>) -> Rect,
 
-    /// offset in bytes fromthe *const ItemImpl.
+    /// offset in bytes from the *const ItemImpl.
     /// isize::MAX  means None
     #[allow(non_upper_case_globals)]
     #[field_offset(CachedRenderingData)]
     pub cached_rendering_data_offset: usize,
 
     /// We would need max/min/preferred size, and all layout info
-    pub layouting_info:
-        extern "C" fn(core::pin::Pin<VRef<ItemVTable>>, window: &ComponentWindow) -> LayoutInfo,
-
-    pub implicit_size:
-        extern "C" fn(core::pin::Pin<VRef<ItemVTable>>, window: &ComponentWindow) -> Size,
+    pub layouting_info: extern "C" fn(
+        core::pin::Pin<VRef<ItemVTable>>,
+        orientation: Orientation,
+        window: &ComponentWindow,
+    ) -> LayoutInfo,
 
     /// Event handler for mouse and touch event. This function is called before being called on children.
     /// Then, depending on the return value, it is called for the children, and their children, then
-    /// [`Self::input_event`] is called on the children, and finaly [`Self::input_event`] is called
+    /// [`Self::input_event`] is called on the children, and finally [`Self::input_event`] is called
     /// on this item again.
     pub input_event_filter_before_children: extern "C" fn(
         core::pin::Pin<VRef<ItemVTable>>,
@@ -194,12 +208,12 @@ impl Item for Rectangle {
         euclid::rect(self.x(), self.y(), self.width(), self.height())
     }
 
-    fn layouting_info(self: Pin<&Self>, _window: &ComponentWindow) -> LayoutInfo {
-        LayoutInfo { horizontal_stretch: 1., vertical_stretch: 1., ..LayoutInfo::default() }
-    }
-
-    fn implicit_size(self: Pin<&Self>, _window: &ComponentWindow) -> Size {
-        Default::default()
+    fn layouting_info(
+        self: Pin<&Self>,
+        _orientation: Orientation,
+        _window: &ComponentWindow,
+    ) -> LayoutInfo {
+        LayoutInfo { stretch: 1., ..LayoutInfo::default() }
     }
 
     fn input_event_filter_before_children(
@@ -238,10 +252,8 @@ impl ItemConsts for Rectangle {
     > = Rectangle::FIELD_OFFSETS.cached_rendering_data.as_unpinned_projection();
 }
 
-ItemVTable_static! {
-    /// The VTable for `Rectangle`
-    #[no_mangle]
-    pub static RectangleVTable for Rectangle
+declare_item_vtable! {
+    fn sixtyfps_get_RectangleVTable() -> RectangleVTable for Rectangle
 }
 
 #[repr(C)]
@@ -267,12 +279,12 @@ impl Item for BorderRectangle {
         euclid::rect(self.x(), self.y(), self.width(), self.height())
     }
 
-    fn layouting_info(self: Pin<&Self>, _window: &ComponentWindow) -> LayoutInfo {
-        LayoutInfo { horizontal_stretch: 1., vertical_stretch: 1., ..LayoutInfo::default() }
-    }
-
-    fn implicit_size(self: Pin<&Self>, _window: &ComponentWindow) -> Size {
-        Default::default()
+    fn layouting_info(
+        self: Pin<&Self>,
+        _orientation: Orientation,
+        _window: &ComponentWindow,
+    ) -> LayoutInfo {
+        LayoutInfo { stretch: 1., ..LayoutInfo::default() }
     }
 
     fn input_event_filter_before_children(
@@ -311,22 +323,8 @@ impl ItemConsts for BorderRectangle {
     > = BorderRectangle::FIELD_OFFSETS.cached_rendering_data.as_unpinned_projection();
 }
 
-ItemVTable_static! {
-    /// The VTable for `BorderRectangle`
-    #[no_mangle]
-    pub static BorderRectangleVTable for BorderRectangle
-}
-
-ItemVTable_static! {
-    /// The VTable for `Image`
-    #[no_mangle]
-    pub static ImageVTable for Image
-}
-
-ItemVTable_static! {
-    /// The VTable for `ClippedImage`
-    #[no_mangle]
-    pub static ClippedImageVTable for ClippedImage
+declare_item_vtable! {
+    fn sixtyfps_get_BorderRectangleVTable() -> BorderRectangleVTable for BorderRectangle
 }
 
 /// The implementation of the `TouchArea` element
@@ -362,14 +360,14 @@ impl Item for TouchArea {
         euclid::rect(self.x(), self.y(), self.width(), self.height())
     }
 
-    fn layouting_info(self: Pin<&Self>, _window: &ComponentWindow) -> LayoutInfo {
+    fn layouting_info(
+        self: Pin<&Self>,
+        _orientation: Orientation,
+        _window: &ComponentWindow,
+    ) -> LayoutInfo {
         LayoutInfo::default()
     }
 
-    fn implicit_size(self: Pin<&Self>, _window: &ComponentWindow) -> Size {
-        Default::default()
-    }
-
     fn input_event_filter_before_children(
         self: Pin<&Self>,
         event: MouseEvent,
@@ -379,9 +377,11 @@ impl Item for TouchArea {
         if !self.enabled() {
             return InputEventFilterResult::ForwardAndIgnore;
         }
-        Self::FIELD_OFFSETS.mouse_x.apply_pin(self).set(event.pos.x);
-        Self::FIELD_OFFSETS.mouse_y.apply_pin(self).set(event.pos.y);
-        Self::FIELD_OFFSETS.has_hover.apply_pin(self).set(event.what != MouseEventType::MouseExit);
+        if let Some(pos) = event.pos() {
+            Self::FIELD_OFFSETS.mouse_x.apply_pin(self).set(pos.x);
+            Self::FIELD_OFFSETS.mouse_y.apply_pin(self).set(pos.y);
+        }
+        Self::FIELD_OFFSETS.has_hover.apply_pin(self).set(!matches!(event, MouseEvent::MouseExit));
         InputEventFilterResult::ForwardAndInterceptGrab
     }
 
@@ -391,27 +391,27 @@ impl Item for TouchArea {
         _window: &ComponentWindow,
         _self_rc: &ItemRc,
     ) -> InputEventResult {
-        if event.what == MouseEventType::MouseExit {
+        if matches!(event, MouseEvent::MouseExit) {
             Self::FIELD_OFFSETS.has_hover.apply_pin(self).set(false)
         }
         if !self.enabled() {
             return InputEventResult::EventIgnored;
         }
-        let result = if matches!(event.what, MouseEventType::MouseReleased) {
+        let result = if matches!(event, MouseEvent::MouseReleased { .. }) {
             Self::FIELD_OFFSETS.clicked.apply_pin(self).call(&());
             InputEventResult::EventAccepted
         } else {
             InputEventResult::GrabMouse
         };
 
-        Self::FIELD_OFFSETS.pressed.apply_pin(self).set(match event.what {
-            MouseEventType::MousePressed => {
-                Self::FIELD_OFFSETS.pressed_x.apply_pin(self).set(event.pos.x);
-                Self::FIELD_OFFSETS.pressed_y.apply_pin(self).set(event.pos.y);
+        Self::FIELD_OFFSETS.pressed.apply_pin(self).set(match event {
+            MouseEvent::MousePressed { pos } => {
+                Self::FIELD_OFFSETS.pressed_x.apply_pin(self).set(pos.x);
+                Self::FIELD_OFFSETS.pressed_y.apply_pin(self).set(pos.y);
                 true
             }
-            MouseEventType::MouseExit | MouseEventType::MouseReleased => false,
-            MouseEventType::MouseMoved => {
+            MouseEvent::MouseExit | MouseEvent::MouseReleased { .. } => false,
+            MouseEvent::MouseMoved { .. } | MouseEvent::MouseWheel { .. } => {
                 return if self.pressed() {
                     InputEventResult::GrabMouse
                 } else {
@@ -438,10 +438,8 @@ impl ItemConsts for TouchArea {
     > = TouchArea::FIELD_OFFSETS.cached_rendering_data.as_unpinned_projection();
 }
 
-ItemVTable_static! {
-    /// The VTable for `TouchArea`
-    #[no_mangle]
-    pub static TouchAreaVTable for TouchArea
+declare_item_vtable! {
+    fn sixtyfps_get_TouchAreaVTable() -> TouchAreaVTable for TouchArea
 }
 
 #[derive(Copy, Clone, Debug, PartialEq, strum_macros::EnumString, strum_macros::Display)]
@@ -482,14 +480,14 @@ impl Item for FocusScope {
         euclid::rect(self.x(), self.y(), self.width(), self.height())
     }
 
-    fn layouting_info(self: Pin<&Self>, _window: &ComponentWindow) -> LayoutInfo {
+    fn layouting_info(
+        self: Pin<&Self>,
+        _orientation: Orientation,
+        _window: &ComponentWindow,
+    ) -> LayoutInfo {
         LayoutInfo::default()
     }
 
-    fn implicit_size(self: Pin<&Self>, _window: &ComponentWindow) -> Size {
-        Default::default()
-    }
-
     fn input_event_filter_before_children(
         self: Pin<&Self>,
         _: MouseEvent,
@@ -508,7 +506,7 @@ impl Item for FocusScope {
         /*if !self.enabled() {
             return InputEventResult::EventIgnored;
         }*/
-        if matches!(event.what, MouseEventType::MousePressed) {
+        if matches!(event, MouseEvent::MousePressed { .. }) {
             if !self.has_focus() {
                 window.set_focus_item(self_rc);
             }
@@ -552,10 +550,8 @@ impl ItemConsts for FocusScope {
     > = FocusScope::FIELD_OFFSETS.cached_rendering_data.as_unpinned_projection();
 }
 
-ItemVTable_static! {
-    /// The VTable for `FocusScope`
-    #[no_mangle]
-    pub static FocusScopeVTable for FocusScope
+declare_item_vtable! {
+    fn sixtyfps_get_FocusScopeVTable() -> FocusScopeVTable for FocusScope
 }
 
 #[repr(C)]
@@ -579,12 +575,12 @@ impl Item for Clip {
         euclid::rect(self.x(), self.y(), self.width(), self.height())
     }
 
-    fn layouting_info(self: Pin<&Self>, _window: &ComponentWindow) -> LayoutInfo {
-        LayoutInfo { horizontal_stretch: 1., vertical_stretch: 1., ..LayoutInfo::default() }
-    }
-
-    fn implicit_size(self: Pin<&Self>, _window: &ComponentWindow) -> Size {
-        Default::default()
+    fn layouting_info(
+        self: Pin<&Self>,
+        _orientation: Orientation,
+        _window: &ComponentWindow,
+    ) -> LayoutInfo {
+        LayoutInfo { stretch: 1., ..LayoutInfo::default() }
     }
 
     fn input_event_filter_before_children(
@@ -626,10 +622,8 @@ impl ItemConsts for Clip {
         Clip::FIELD_OFFSETS.cached_rendering_data.as_unpinned_projection();
 }
 
-ItemVTable_static! {
-    /// The VTable for `Clip`
-    #[no_mangle]
-    pub static ClipVTable for Clip
+declare_item_vtable! {
+    fn sixtyfps_get_ClipVTable() -> ClipVTable for Clip
 }
 
 #[repr(C)]
@@ -653,12 +647,12 @@ impl Item for Opacity {
         euclid::rect(self.x(), self.y(), self.width(), self.height())
     }
 
-    fn layouting_info(self: Pin<&Self>, _window: &ComponentWindow) -> LayoutInfo {
-        LayoutInfo { horizontal_stretch: 1., vertical_stretch: 1., ..LayoutInfo::default() }
-    }
-
-    fn implicit_size(self: Pin<&Self>, _window: &ComponentWindow) -> Size {
-        Default::default()
+    fn layouting_info(
+        self: Pin<&Self>,
+        _orientation: Orientation,
+        _window: &ComponentWindow,
+    ) -> LayoutInfo {
+        LayoutInfo { stretch: 1., ..LayoutInfo::default() }
     }
 
     fn input_event_filter_before_children(
@@ -697,10 +691,8 @@ impl ItemConsts for Opacity {
     > = Opacity::FIELD_OFFSETS.cached_rendering_data.as_unpinned_projection();
 }
 
-ItemVTable_static! {
-    /// The VTable for `Opacity`
-    #[no_mangle]
-    pub static OpacityVTable for Opacity
+declare_item_vtable! {
+    fn sixtyfps_get_OpacityVTable() -> OpacityVTable for Opacity
 }
 
 #[repr(C)]
@@ -723,12 +715,12 @@ impl Item for Rotate {
         euclid::rect(0., 0., 0., 0.)
     }
 
-    fn layouting_info(self: Pin<&Self>, _window: &ComponentWindow) -> LayoutInfo {
-        LayoutInfo { horizontal_stretch: 1., vertical_stretch: 1., ..LayoutInfo::default() }
-    }
-
-    fn implicit_size(self: Pin<&Self>, _window: &ComponentWindow) -> Size {
-        Default::default()
+    fn layouting_info(
+        self: Pin<&Self>,
+        _orientation: Orientation,
+        _window: &ComponentWindow,
+    ) -> LayoutInfo {
+        LayoutInfo { stretch: 1., ..LayoutInfo::default() }
     }
 
     fn input_event_filter_before_children(
@@ -769,10 +761,8 @@ impl ItemConsts for Rotate {
     > = Rotate::FIELD_OFFSETS.cached_rendering_data.as_unpinned_projection();
 }
 
-ItemVTable_static! {
-    /// The VTable for `Rotate`
-    #[no_mangle]
-    pub static RotateVTable for Rotate
+declare_item_vtable! {
+    fn sixtyfps_get_RotateVTable() -> RotateVTable for Rotate
 }
 
 #[derive(Copy, Clone, Debug, PartialEq, strum_macros::EnumString, strum_macros::Display)]
@@ -803,6 +793,11 @@ pub struct Path {
     pub fill_rule: Property<FillRule>,
     pub stroke: Property<Brush>,
     pub stroke_width: Property<f32>,
+    pub viewbox_x: Property<f32>,
+    pub viewbox_y: Property<f32>,
+    pub viewbox_width: Property<f32>,
+    pub viewbox_height: Property<f32>,
+    pub clip: Property<bool>,
     pub cached_rendering_data: CachedRenderingData,
 }
 
@@ -810,17 +805,17 @@ impl Item for Path {
     fn init(self: Pin<&Self>, _window: &ComponentWindow) {}
 
     fn geometry(self: Pin<&Self>) -> Rect {
-        euclid::rect(self.x(), self.y(), 0., 0.)
+        euclid::rect(self.x(), self.y(), self.width(), self.height())
     }
 
-    fn layouting_info(self: Pin<&Self>, _window: &ComponentWindow) -> LayoutInfo {
+    fn layouting_info(
+        self: Pin<&Self>,
+        _orientation: Orientation,
+        _window: &ComponentWindow,
+    ) -> LayoutInfo {
         LayoutInfo::default()
     }
 
-    fn implicit_size(self: Pin<&Self>, _window: &ComponentWindow) -> Size {
-        Default::default()
-    }
-
     fn input_event_filter_before_children(
         self: Pin<&Self>,
         _: MouseEvent,
@@ -846,7 +841,15 @@ impl Item for Path {
     fn focus_event(self: Pin<&Self>, _: &FocusEvent, _window: &ComponentWindow) {}
 
     fn render(self: Pin<&Self>, backend: &mut ItemRendererRef) {
-        (*backend).draw_path(self)
+        let clip = self.clip();
+        if clip {
+            (*backend).save_state();
+            (*backend).combine_clip(self.geometry(), 0., 0.)
+        }
+        (*backend).draw_path(self);
+        if clip {
+            (*backend).restore_state();
+        }
     }
 }
 
@@ -861,8 +864,20 @@ impl Path {
         let bounds_width = (self.width() - stroke_width).max(0.);
         let bounds_height = (self.height() - stroke_width).max(0.);
         let offset = euclid::default::Vector2D::new(stroke_width / 2., stroke_width / 2.);
-        let event_iterator = self.elements().iter_fitted(bounds_width, bounds_height);
-        (offset, event_iterator)
+
+        let viewbox_width = self.viewbox_width();
+        let viewbox_height = self.viewbox_height();
+
+        let mut elements_iter = self.elements().iter();
+
+        let maybe_viewbox = if viewbox_width > 0. && viewbox_height > 0. {
+            Some(euclid::rect(self.viewbox_x(), self.viewbox_y(), viewbox_width, viewbox_height))
+        } else {
+            None
+        };
+
+        elements_iter.fit(bounds_width, bounds_height, maybe_viewbox);
+        (offset, elements_iter)
     }
 }
 
@@ -871,10 +886,8 @@ impl ItemConsts for Path {
         Path::FIELD_OFFSETS.cached_rendering_data.as_unpinned_projection();
 }
 
-ItemVTable_static! {
-    /// The VTable for `Path`
-    #[no_mangle]
-    pub static PathVTable for Path
+declare_item_vtable! {
+    fn sixtyfps_get_PathVTable() -> PathVTable for Path
 }
 
 /// The implementation of the `Flickable` element
@@ -901,12 +914,12 @@ impl Item for Flickable {
         euclid::rect(self.x(), self.y(), self.width(), self.height())
     }
 
-    fn layouting_info(self: Pin<&Self>, _window: &ComponentWindow) -> LayoutInfo {
-        LayoutInfo { horizontal_stretch: 1., vertical_stretch: 1., ..LayoutInfo::default() }
-    }
-
-    fn implicit_size(self: Pin<&Self>, _window: &ComponentWindow) -> Size {
-        Default::default()
+    fn layouting_info(
+        self: Pin<&Self>,
+        _orientation: Orientation,
+        _window: &ComponentWindow,
+    ) -> LayoutInfo {
+        LayoutInfo { stretch: 1., ..LayoutInfo::default() }
     }
 
     fn input_event_filter_before_children(
@@ -915,7 +928,7 @@ impl Item for Flickable {
         _window: &ComponentWindow,
         _self_rc: &ItemRc,
     ) -> InputEventFilterResult {
-        if !self.interactive() {
+        if !self.interactive() && !matches!(event, MouseEvent::MouseWheel { .. }) {
             return InputEventFilterResult::ForwardAndIgnore;
         }
         self.data.handle_mouse_filter(self, event)
@@ -927,7 +940,7 @@ impl Item for Flickable {
         _window: &ComponentWindow,
         _self_rc: &ItemRc,
     ) -> InputEventResult {
-        if !self.interactive() {
+        if !self.interactive() && !matches!(event, MouseEvent::MouseWheel { .. }) {
             return InputEventResult::EventIgnored;
         }
         self.data.handle_mouse(self, event)
@@ -950,10 +963,8 @@ impl ItemConsts for Flickable {
         Self::FIELD_OFFSETS.cached_rendering_data.as_unpinned_projection();
 }
 
-ItemVTable_static! {
-    /// The VTable for `Flickable`
-    #[no_mangle]
-    pub static FlickableVTable for Flickable
+declare_item_vtable! {
+    fn sixtyfps_get_FlickableVTable() -> FlickableVTable for Flickable
 }
 
 pub use crate::SharedVector;
@@ -989,7 +1000,7 @@ pub unsafe extern "C" fn sixtyfps_flickable_data_init(data: *mut FlickableDataBo
 }
 #[no_mangle]
 pub unsafe extern "C" fn sixtyfps_flickable_data_free(data: *mut FlickableDataBox) {
-    std::ptr::read(data);
+    std::ptr::drop_in_place(data);
 }
 
 /// The implementation of the `PropertyAnimation` element
@@ -1027,14 +1038,14 @@ impl Item for Window {
         euclid::rect(0., 0., self.width(), self.height())
     }
 
-    fn layouting_info(self: Pin<&Self>, _window: &ComponentWindow) -> LayoutInfo {
+    fn layouting_info(
+        self: Pin<&Self>,
+        _orientation: Orientation,
+        _window: &ComponentWindow,
+    ) -> LayoutInfo {
         LayoutInfo::default()
     }
 
-    fn implicit_size(self: Pin<&Self>, _window: &ComponentWindow) -> Size {
-        Default::default()
-    }
-
     fn input_event_filter_before_children(
         self: Pin<&Self>,
         _: MouseEvent,
@@ -1100,10 +1111,8 @@ impl ItemConsts for Window {
         Self::FIELD_OFFSETS.cached_rendering_data.as_unpinned_projection();
 }
 
-ItemVTable_static! {
-    /// The VTable for `Window`
-    #[no_mangle]
-    pub static WindowVTable for Window
+declare_item_vtable! {
+    fn sixtyfps_get_WindowVTable() -> WindowVTable for Window
 }
 
 /// The implementation of the `BoxShadow` element
@@ -1132,12 +1141,12 @@ impl Item for BoxShadow {
         euclid::rect(self.x(), self.y(), self.width(), self.height())
     }
 
-    fn layouting_info(self: Pin<&Self>, _window: &ComponentWindow) -> LayoutInfo {
-        LayoutInfo { horizontal_stretch: 1., vertical_stretch: 1., ..LayoutInfo::default() }
-    }
-
-    fn implicit_size(self: Pin<&Self>, _window: &ComponentWindow) -> Size {
-        Default::default()
+    fn layouting_info(
+        self: Pin<&Self>,
+        _orientation: Orientation,
+        _window: &ComponentWindow,
+    ) -> LayoutInfo {
+        LayoutInfo { stretch: 1., ..LayoutInfo::default() }
     }
 
     fn input_event_filter_before_children(
@@ -1174,20 +1183,22 @@ impl ItemConsts for BoxShadow {
         Self::FIELD_OFFSETS.cached_rendering_data.as_unpinned_projection();
 }
 
-ItemVTable_static! {
-    /// The VTable for `BoxShadow`
-    #[no_mangle]
-    pub static BoxShadowVTable for BoxShadow
+declare_item_vtable! {
+    fn sixtyfps_get_BoxShadowVTable() -> BoxShadowVTable for BoxShadow
 }
 
-ItemVTable_static! {
-    /// The VTable for `Text`
-    #[no_mangle]
-    pub static TextVTable for Text
+declare_item_vtable! {
+    fn sixtyfps_get_TextVTable() -> TextVTable for Text
 }
 
-ItemVTable_static! {
-    /// The VTable for `TextInput`
-    #[no_mangle]
-    pub static TextInputVTable for TextInput
+declare_item_vtable! {
+    fn sixtyfps_get_TextInputVTable() -> TextInputVTable for TextInput
+}
+
+declare_item_vtable! {
+    fn sixtyfps_get_ImageItemVTable() -> ImageItemVTable for ImageItem
+}
+
+declare_item_vtable! {
+    fn sixtyfps_get_ClippedImageVTable() -> ClippedImageVTable for ClippedImage
 }
