@@ -11,24 +11,18 @@ LICENSE END */
 This module contains the builtin text related items.
 
 When adding an item or a property, it needs to be kept in sync with different place.
-(This is less than ideal and maybe we can have some automation later)
-
- - It needs to be changed in this module
- - In the compiler: builtins.60
- - In the interpreter: dynamic_component.rs
- - For the C++ code (new item only): the cbindgen.rs to export the new item, and the `using` declaration in sixtyfps.h
- - Don't forget to update the documentation
+Lookup the [`crate::items`] module documentation.
 */
 
 use super::{Item, ItemConsts, ItemRc, VoidArg};
-use crate::graphics::{Brush, Color, FontRequest, Rect, Size};
+use crate::graphics::{Brush, Color, FontRequest, Rect};
 use crate::input::{
     FocusEvent, InputEventResult, KeyEvent, KeyEventResult, KeyEventType, KeyboardModifiers,
-    MouseEvent, MouseEventType,
+    MouseEvent,
 };
 use crate::input::{InputEventFilterResult, InternalKeyCode};
 use crate::item_rendering::{CachedRenderingData, ItemRenderer};
-use crate::layout::LayoutInfo;
+use crate::layout::{LayoutInfo, Orientation};
 #[cfg(feature = "rtti")]
 use crate::rtti::*;
 use crate::window::ComponentWindow;
@@ -124,45 +118,48 @@ impl Item for Text {
         euclid::rect(self.x(), self.y(), self.width(), self.height())
     }
 
-    fn layouting_info(self: Pin<&Self>, window: &ComponentWindow) -> LayoutInfo {
-        if self.wrap() == TextWrap::word_wrap {
-            // FIXME: one should limit to the size of the smaler word
-            LayoutInfo::default()
-        } else if let Some(font_metrics) = window.0.font_metrics(
+    fn layouting_info(
+        self: Pin<&Self>,
+        orientation: Orientation,
+        window: &ComponentWindow,
+    ) -> LayoutInfo {
+        let font_metrics = window.0.font_metrics(
             &self.cached_rendering_data,
             &|| self.unresolved_font_request(),
             Self::FIELD_OFFSETS.text.apply_pin(self),
-        ) {
-            let mut min_size = font_metrics.text_size(&self.text());
-            match self.overflow() {
-                TextOverflow::elide => {
-                    min_size.width = font_metrics.text_size("…").width;
-                }
-                TextOverflow::clip => {}
-            }
-            // Stretch uses `round_layout` to explicitly align the top left and bottom right of layout nodes
-            // to pixel boundaries. To avoid rounding down causing the minimum width to become so little that
-            // letters will be cut off, apply the ceiling here.
-            LayoutInfo {
-                min_width: min_size.width.ceil(),
-                min_height: min_size.height.ceil(),
-                ..LayoutInfo::default()
-            }
-        } else {
-            LayoutInfo::default()
-        }
-    }
+        );
+        let text = self.text();
+        let implicit_size = font_metrics.text_size(&text, None);
 
-    fn implicit_size(self: Pin<&Self>, window: &ComponentWindow) -> Size {
-        window
-            .0
-            .font_metrics(
-                &self.cached_rendering_data,
-                &|| self.unresolved_font_request(),
-                Self::FIELD_OFFSETS.text.apply_pin(self),
-            )
-            .map(|metrics| metrics.text_size(&self.text()).ceil())
-            .unwrap_or_default()
+        // Stretch uses `round_layout` to explicitly align the top left and bottom right of layout nodes
+        // to pixel boundaries. To avoid rounding down causing the minimum width to become so little that
+        // letters will be cut off, apply the ceiling here.
+        match orientation {
+            Orientation::Horizontal => {
+                let min = match self.overflow() {
+                    TextOverflow::elide => {
+                        implicit_size.width.min(font_metrics.text_size("…", None).width)
+                    }
+                    TextOverflow::clip => match self.wrap() {
+                        TextWrap::no_wrap => implicit_size.width,
+                        TextWrap::word_wrap => 0.,
+                    },
+                };
+                LayoutInfo {
+                    min: min.ceil(),
+                    preferred: implicit_size.width.ceil(),
+                    ..LayoutInfo::default()
+                }
+            }
+            Orientation::Vertical => {
+                let h = match self.wrap() {
+                    TextWrap::no_wrap => implicit_size.height,
+                    TextWrap::word_wrap => font_metrics.text_size(&text, Some(self.width())).height,
+                }
+                .ceil();
+                LayoutInfo { min: h, preferred: h, ..LayoutInfo::default() }
+            }
+        }
     }
 
     fn input_event_filter_before_children(
@@ -270,37 +267,31 @@ impl Item for TextInput {
         euclid::rect(self.x(), self.y(), self.width(), self.height())
     }
 
-    fn layouting_info(self: Pin<&Self>, window: &ComponentWindow) -> LayoutInfo {
-        if let Some(font_metrics) = window.0.font_metrics(
+    fn layouting_info(
+        self: Pin<&Self>,
+        orientation: Orientation,
+        window: &ComponentWindow,
+    ) -> LayoutInfo {
+        let font_metrics = window.0.font_metrics(
             &self.cached_rendering_data,
             &|| self.unresolved_font_request(),
             Self::FIELD_OFFSETS.text.apply_pin(self),
-        ) {
-            let size = font_metrics.text_size("********************");
+        );
+        let size = font_metrics.text_size("********************", None).ceil();
 
-            LayoutInfo {
-                min_width: size.width,
-                min_height: size.height,
-                horizontal_stretch: 1.,
+        match orientation {
+            Orientation::Horizontal => LayoutInfo {
+                min: size.width,
+                preferred: size.width,
+                stretch: 1.,
                 ..LayoutInfo::default()
+            },
+            Orientation::Vertical => {
+                LayoutInfo { min: size.height, preferred: size.height, ..LayoutInfo::default() }
             }
-        } else {
-            LayoutInfo::default()
         }
     }
 
-    fn implicit_size(self: Pin<&Self>, window: &ComponentWindow) -> Size {
-        window
-            .0
-            .font_metrics(
-                &self.cached_rendering_data,
-                &|| self.unresolved_font_request(),
-                Self::FIELD_OFFSETS.text.apply_pin(self),
-            )
-            .map(|metrics| metrics.text_size(&self.text()).ceil())
-            .unwrap_or_default()
-    }
-
     fn input_event_filter_before_children(
         self: Pin<&Self>,
         _: MouseEvent,
@@ -321,35 +312,33 @@ impl Item for TextInput {
         }
 
         let text = self.text();
-        let font_metrics = match window.0.font_metrics(
+        let font_metrics = window.0.font_metrics(
             &self.cached_rendering_data,
             &|| self.unresolved_font_request(),
             Self::FIELD_OFFSETS.text.apply_pin(self),
-        ) {
-            Some(font) => font,
-            None => return InputEventResult::EventIgnored,
-        };
-        let clicked_offset = font_metrics.text_offset_for_x_position(&text, event.pos.x) as i32;
-
-        if matches!(event.what, MouseEventType::MousePressed) {
-            self.as_ref().pressed.set(true);
-            self.as_ref().anchor_position.set(clicked_offset);
-            self.as_ref().cursor_position.set(clicked_offset);
-            if !self.has_focus() {
-                window.set_focus_item(self_rc);
-            }
-        }
-
-        match event.what {
-            MouseEventType::MouseReleased => {
-                self.as_ref().pressed.set(false);
-            }
-            MouseEventType::MouseMoved if self.as_ref().pressed.get() => {
+        );
+        match event {
+            MouseEvent::MousePressed { pos } => {
+                let clicked_offset = font_metrics.text_offset_for_x_position(&text, pos.x) as i32;
+                self.as_ref().pressed.set(true);
+                self.as_ref().anchor_position.set(clicked_offset);
                 self.as_ref().cursor_position.set(clicked_offset);
+                if !self.has_focus() {
+                    window.set_focus_item(self_rc);
+                }
             }
-            _ => {}
+            MouseEvent::MouseReleased { .. } | MouseEvent::MouseExit => {
+                self.as_ref().pressed.set(false)
+            }
+            MouseEvent::MouseMoved { pos } => {
+                if self.as_ref().pressed.get() {
+                    let clicked_offset =
+                        font_metrics.text_offset_for_x_position(&text, pos.x) as i32;
+                    self.as_ref().cursor_position.set(clicked_offset);
+                }
+            }
+            MouseEvent::MouseWheel { .. } => return InputEventResult::EventIgnored,
         }
-
         InputEventResult::EventAccepted
     }
 
