@@ -17,7 +17,9 @@ use std::{
     rc::{Rc, Weak},
 };
 
+use crate::item_tree::TraversalOrder;
 use crate::items::ItemRef;
+use crate::layout::Orientation;
 use crate::Property;
 
 type ModelPeerInner = dyn ViewAbstraction;
@@ -285,7 +287,10 @@ pub trait RepeatedComponent: crate::component::Component {
     }
 
     /// Returns what's needed to perform the layout if this component is in a box layout
-    fn box_layout_data<'a>(self: Pin<&'a Self>) -> crate::layout::BoxLayoutCellData<'a> {
+    fn box_layout_data<'a>(
+        self: Pin<&'a Self>,
+        _orientation: Orientation,
+    ) -> crate::layout::BoxLayoutCellData {
         crate::layout::BoxLayoutCellData::default()
     }
 }
@@ -493,17 +498,20 @@ impl<C: RepeatedComponent + 'static> Repeater<C> {
         let init = &init;
         if listview_geometry_tracker
             .evaluate_if_dirty(|| {
+                let listview_height = listview_height.get();
                 // Compute the element height
                 let total_height = Cell::new(0.);
+                let min_height = Cell::new(listview_height);
                 let count = Cell::new(0);
 
                 let get_height_visitor = |item: Pin<ItemRef>| {
                     count.set(count.get() + 1);
-                    total_height.set(total_height.get() + item.as_ref().geometry().height());
+                    let height = item.as_ref().geometry().height();
+                    total_height.set(total_height.get() + height);
+                    min_height.set(min_height.get().min(height))
                 };
                 for c in self.inner.borrow().borrow().components.iter() {
                     c.1.as_ref().map(|x| {
-                        x.as_pin_ref().apply_layout(Default::default());
                         get_height_visitor(x.as_pin_ref().get_item_ref(0));
                     });
                 }
@@ -522,7 +530,6 @@ impl<C: RepeatedComponent + 'static> Repeater<C> {
                     self.ensure_updated_impl(&init, &model, 1);
                     if let Some(c) = self.inner.borrow().borrow().components.get(0) {
                         c.1.as_ref().map(|x| {
-                            x.as_pin_ref().apply_layout(Default::default());
                             get_height_visitor(x.as_pin_ref().get_item_ref(0));
                         });
                     } else {
@@ -531,18 +538,28 @@ impl<C: RepeatedComponent + 'static> Repeater<C> {
                     total_height.get()
                 };
 
-                let listview_height = listview_height.get();
+                let min_height = min_height.get().min(element_height).max(1.);
+
                 viewport_height.set(element_height * row_count as f32);
                 self.inner.borrow().borrow_mut().cached_item_height = element_height;
                 if -viewport_y.get() > element_height * row_count as f32 - listview_height {
                     viewport_y.set(-(element_height * row_count as f32 - listview_height).max(0.))
                 }
-                let offset = (-viewport_y.get() / element_height).floor() as usize;
-                let count =
-                    ((listview_height / element_height).ceil() as usize).min(row_count - offset);
-                self.set_offset(offset, count);
-                self.ensure_updated_impl(init, &model, count);
-                self.compute_layout_listview(viewport_width, listview_width);
+                let offset = (-viewport_y.get_untracked() / element_height).floor() as usize;
+                let mut count =
+                    ((listview_height / min_height).ceil() as usize).min(row_count - offset);
+                loop {
+                    self.set_offset(offset, count);
+                    self.ensure_updated_impl(init, &model, count);
+                    let end = self.compute_layout_listview(viewport_width, listview_width);
+                    let diff = listview_height - viewport_y.get_untracked() - end;
+                    if diff > 0. && count < row_count - offset {
+                        count = (count + (diff / element_height).ceil() as usize)
+                            .min(row_count - offset);
+                        continue;
+                    }
+                    break;
+                }
             })
             .is_none()
         {
@@ -605,12 +622,13 @@ impl<C: RepeatedComponent> Repeater<C> {
     /// Call the visitor for each component
     pub fn visit(
         &self,
-        order: crate::item_tree::TraversalOrder,
+        order: TraversalOrder,
         mut visitor: crate::item_tree::ItemVisitorRefMut,
     ) -> crate::item_tree::VisitChildrenResult {
         // We can't keep self.inner borrowed because the event might modify the model
         let count = self.inner.borrow().borrow().components.len();
         for i in 0..count {
+            let i = if order == TraversalOrder::BackToFront { i } else { count - i - 1 };
             let c = self.inner.borrow().borrow().components.get(i).and_then(|c| c.1.clone());
             if let Some(c) = c {
                 if c.as_pin_ref().visit_children_item(-1, order, visitor.borrow_mut()).has_aborted()
@@ -632,19 +650,14 @@ impl<C: RepeatedComponent> Repeater<C> {
         self.inner.borrow().borrow().components.iter().flat_map(|x| x.1.clone()).collect()
     }
 
-    /// Recompute the layout of each child elements
-    pub fn compute_layout(&self) {
-        for c in self.inner.borrow().borrow().components.iter() {
-            c.1.as_ref().map(|x| x.as_pin_ref().apply_layout(Default::default()));
-        }
-    }
-
-    /// Same as compule_layout, but only for the ListView
+    /// Set the position of all the element in the listview
+    ///
+    /// Returns the offset of the end of the last element
     pub fn compute_layout_listview(
         &self,
         viewport_width: Pin<&Property<f32>>,
         listview_width: f32,
-    ) {
+    ) -> f32 {
         let inner = self.inner.borrow();
         let inner = inner.borrow();
         let mut y_offset = inner.offset as f32 * inner.cached_item_height;
@@ -652,6 +665,7 @@ impl<C: RepeatedComponent> Repeater<C> {
         for c in self.inner.borrow().borrow().components.iter() {
             c.1.as_ref().map(|x| x.as_pin_ref().listview_layout(&mut y_offset, viewport_width));
         }
+        y_offset
     }
 }
 
