@@ -102,7 +102,7 @@ use crate::items::PropertyAnimation;
 /// The return value of a binding
 #[derive(Copy, Clone, Debug, Eq, PartialEq)]
 enum BindingResult {
-    /// The binding is a normal binding, and we keep it to re-evaluate it ince it is dirty
+    /// The binding is a normal binding, and we keep it to re-evaluate it once it is dirty
     KeepBinding,
     /// The value of the property is now constant after the binding was evaluated, so
     /// the binding can be removed.
@@ -249,6 +249,7 @@ impl DependencyListHead {
     unsafe fn mem_move(from: *mut Self, to: *mut Self) {
         (*to).0.set((*from).0.get());
         if let Some(next) = ((*from).0.get() as *const DependencyNode).as_ref() {
+            debug_assert_eq!(from as *const _, next.prev.get() as *const _);
             next.debug_assert_valid();
             next.prev.set(to as *const _);
             next.debug_assert_valid();
@@ -256,6 +257,7 @@ impl DependencyListHead {
     }
     unsafe fn drop(_self: *mut Self) {
         if let Some(next) = ((*_self).0.get() as *const DependencyNode).as_ref() {
+            debug_assert_eq!(_self as *const _, next.prev.get() as *const _);
             next.debug_assert_valid();
             next.prev.set(core::ptr::null());
             next.debug_assert_valid();
@@ -320,6 +322,7 @@ impl DependencyNode {
                 prev.set(self.next.get());
             }
             if let Some(next) = self.next.get().as_ref() {
+                next.debug_assert_valid();
                 next.prev.set(self.prev.get());
                 next.debug_assert_valid();
             }
@@ -342,7 +345,7 @@ struct PropertyHandle {
     /// The two least significant bit of the pointer are flags, as the pointer will be aligned.
     /// The least significant bit (`0b01`) tells that the binding is borrowed. So no two reference to the
     /// binding exist at the same time.
-    /// The decond to last bit (`0b10`) tells that the pointer points to a binding. Otherwise, it is the head
+    /// The second to last bit (`0b10`) tells that the pointer points to a binding. Otherwise, it is the head
     /// node of the linked list of dependent binding
     handle: Cell<usize>,
 }
@@ -353,7 +356,7 @@ impl PropertyHandle {
         self.handle.get() & 0b1 == 1
     }
     /// Sets the lock_flag.
-    /// Safety: the lock flag must not be unsat if there exist reference to what's inside the cell
+    /// Safety: the lock flag must not be unset if there exist reference to what's inside the cell
     unsafe fn set_lock_flag(&self, set: bool) {
         self.handle.set(if set { self.handle.get() | 0b1 } else { self.handle.get() & !0b1 })
     }
@@ -456,7 +459,7 @@ impl PropertyHandle {
     }
 
     /// Register this property as a dependency to the current binding being evaluated
-    fn register_as_dependency_to_current_binding(&self) {
+    fn register_as_dependency_to_current_binding(self: Pin<&Self>) {
         if CURRENT_BINDING.is_set() {
             CURRENT_BINDING.with(|cur_binding| {
                 cur_binding.register_self_as_dependency(self.dependencies());
@@ -479,7 +482,7 @@ impl Drop for PropertyHandle {
     }
 }
 
-/// Safety: the dependency list must be valid and consistant
+/// Safety: the dependency list must be valid and consistent
 unsafe fn mark_dependencies_dirty(deps: *mut DependencyListHead) {
     let mut next = (*deps).0.get() as *const DependencyNode;
     while let Some(node) = next.as_ref() {
@@ -555,14 +558,15 @@ impl<T: Clone> Property<T> {
     /// cloning the value.
     pub fn get(self: Pin<&Self>) -> T {
         unsafe { self.handle.update(self.value.get()) };
-        self.handle.register_as_dependency_to_current_binding();
+        let handle = unsafe { Pin::new_unchecked(&self.handle) };
+        handle.register_as_dependency_to_current_binding();
         self.get_internal()
     }
 
     /// Same as get() but without registering a dependency
     ///
     /// This allow to optimize bindings that know that they might not need to
-    /// re_evaluate themself when the property change or that have registered
+    /// re_evaluate themselves when the property change or that have registered
     /// the dependency in another way.
     ///
     /// ## Example
@@ -629,7 +633,7 @@ impl<T: Clone> Property<T> {
     /// If other properties have bindings depending of this property, these properties will
     /// be marked as dirty.
     ///
-    /// Clausures of type `Fn()->T` implements Binding<T> and can be used as a binding
+    /// Closures of type `Fn()->T` implements Binding<T> and can be used as a binding
     ///
     /// ## Example
     /// ```
@@ -660,7 +664,7 @@ impl<T: Clone> Property<T> {
     }
 
     /// Any of the properties accessed during the last evaluation of the closure called
-    /// from the last call to evaluate is pottentially dirty.
+    /// from the last call to evaluate is potentially dirty.
     pub fn is_dirty(&self) -> bool {
         self.handle.access(|binding| binding.map_or(false, |b| b.dirty.get()))
     }
@@ -680,7 +684,7 @@ impl<T: Clone + InterpolatedPropertyValue + 'static> Property<T> {
             value,
             animation_data,
         ));
-        // Safety: the BindingCallable will cast its arguement to T
+        // Safety: the BindingCallable will cast its argument to T
         unsafe {
             self.handle.set_binding(move |val: *mut ()| {
                 let (value, finished) = d.borrow_mut().compute_interpolated_value();
@@ -953,7 +957,8 @@ impl<T: InterpolatedPropertyValue + Clone, A: Fn() -> AnimationDetail> BindingCa
     for AnimatedBindingCallable<T, A>
 {
     unsafe fn evaluate(self: Pin<&Self>, value: *mut ()) -> BindingResult {
-        self.original_binding.register_as_dependency_to_current_binding();
+        let original_binding = Pin::new_unchecked(&self.original_binding);
+        original_binding.register_as_dependency_to_current_binding();
         match self.state.get() {
             AnimatedBindingState::Animating => {
                 let (val, finished) = self.animation_data.borrow_mut().compute_interpolated_value();
@@ -972,6 +977,7 @@ impl<T: InterpolatedPropertyValue + Clone, A: Fn() -> AnimationDetail> BindingCa
                 let value = &mut *(value as *mut T);
                 self.state.set(AnimatedBindingState::Animating);
                 let mut animation_data = self.animation_data.borrow_mut();
+                animation_data.loop_iteration = 0;
                 animation_data.from_value = value.clone();
                 self.original_binding.update((&mut animation_data.to_value) as *mut T as *mut ());
                 if let Some((details, start_time)) = (self.compute_animation_details)() {
@@ -1224,6 +1230,93 @@ mod animation_tests {
         // the binding should be removed
         compo.width.handle.access(|binding| assert!(binding.is_none()));
     }
+
+    #[test]
+    fn test_loop_via_binding() {
+        // Loop twice, restart the animation and still loop twice.
+
+        let compo = Component::new_test_component();
+
+        let start_time = crate::animations::current_tick();
+
+        let animation_details = PropertyAnimation {
+            duration: DURATION.as_millis() as _,
+            loop_count: 1,
+            ..PropertyAnimation::default()
+        };
+
+        let w = Rc::downgrade(&compo);
+        compo.width.set_animated_binding(
+            move || {
+                let compo = w.upgrade().unwrap();
+                get_prop_value(&compo.feed_property)
+            },
+            animation_details,
+        );
+
+        compo.feed_property.set(100);
+        assert_eq!(get_prop_value(&compo.width), 100);
+
+        compo.feed_property.set(200);
+        assert_eq!(get_prop_value(&compo.width), 100);
+
+        crate::animations::CURRENT_ANIMATION_DRIVER
+            .with(|driver| driver.update_animations(start_time + DURATION / 2));
+
+        assert_eq!(get_prop_value(&compo.width), 150);
+
+        crate::animations::CURRENT_ANIMATION_DRIVER
+            .with(|driver| driver.update_animations(start_time + DURATION));
+
+        assert_eq!(get_prop_value(&compo.width), 100);
+
+        crate::animations::CURRENT_ANIMATION_DRIVER
+            .with(|driver| driver.update_animations(start_time + DURATION + DURATION / 2));
+
+        assert_eq!(get_prop_value(&compo.width), 150);
+
+        crate::animations::CURRENT_ANIMATION_DRIVER
+            .with(|driver| driver.update_animations(start_time + 2 * DURATION));
+
+        assert_eq!(get_prop_value(&compo.width), 200);
+
+        crate::animations::CURRENT_ANIMATION_DRIVER
+            .with(|driver| driver.update_animations(start_time + 2 * DURATION + DURATION / 2));
+
+        assert_eq!(get_prop_value(&compo.width), 200);
+
+        // Restart the animation by setting a new value.
+
+        let start_time = crate::animations::current_tick();
+
+        compo.feed_property.set(300);
+        assert_eq!(get_prop_value(&compo.width), 200);
+
+        crate::animations::CURRENT_ANIMATION_DRIVER
+            .with(|driver| driver.update_animations(start_time + DURATION / 2));
+
+        assert_eq!(get_prop_value(&compo.width), 250);
+
+        crate::animations::CURRENT_ANIMATION_DRIVER
+            .with(|driver| driver.update_animations(start_time + DURATION));
+
+        assert_eq!(get_prop_value(&compo.width), 200);
+
+        crate::animations::CURRENT_ANIMATION_DRIVER
+            .with(|driver| driver.update_animations(start_time + DURATION + DURATION / 2));
+
+        assert_eq!(get_prop_value(&compo.width), 250);
+
+        crate::animations::CURRENT_ANIMATION_DRIVER
+            .with(|driver| driver.update_animations(start_time + 2 * DURATION));
+
+        assert_eq!(get_prop_value(&compo.width), 300);
+
+        crate::animations::CURRENT_ANIMATION_DRIVER
+            .with(|driver| driver.update_animations(start_time + 2 * DURATION + DURATION / 2));
+
+        assert_eq!(get_prop_value(&compo.width), 300);
+    }
 }
 
 /// Value of the state property
@@ -1326,7 +1419,7 @@ impl<ChangeHandler> Drop for PropertyTracker<ChangeHandler> {
 
 impl<ChangeHandler: PropertyChangeHandler> PropertyTracker<ChangeHandler> {
     /// Register this property tracker as a dependency to the current binding/property tracker being evaluated
-    fn register_as_dependency_to_current_binding(&self) {
+    fn register_as_dependency_to_current_binding(self: Pin<&Self>) {
         if CURRENT_BINDING.is_set() {
             CURRENT_BINDING.with(|cur_binding| {
                 cur_binding.register_self_as_dependency(
@@ -1337,12 +1430,12 @@ impl<ChangeHandler: PropertyChangeHandler> PropertyTracker<ChangeHandler> {
     }
 
     /// Any of the properties accessed during the last evaluation of the closure called
-    /// from the last call to evaluate is pottentially dirty.
+    /// from the last call to evaluate is potentially dirty.
     pub fn is_dirty(&self) -> bool {
         self.holder.dirty.get()
     }
 
-    /// Evaluate the function, and record dependencies of properties accessed whithin this function.
+    /// Evaluate the function, and record dependencies of properties accessed within this function.
     /// If this is called during the evaluation of another property binding or property tracker, then
     /// any changes to accessed properties will also mark the other binding/tracker dirty.
     pub fn evaluate<R>(self: Pin<&Self>, f: impl FnOnce() -> R) -> R {
@@ -1350,7 +1443,7 @@ impl<ChangeHandler: PropertyChangeHandler> PropertyTracker<ChangeHandler> {
         self.evaluate_as_dependency_root(f)
     }
 
-    /// Evaluate the function, and record dependencies of properties accessed whithin this function.
+    /// Evaluate the function, and record dependencies of properties accessed within this function.
     /// If this is called during the evaluation of another property binding or property tracker, then
     /// any changes to accessed properties will not propagate to the other tracker.
     pub fn evaluate_as_dependency_root<R>(self: Pin<&Self>, f: impl FnOnce() -> R) -> R {
@@ -1578,8 +1671,9 @@ pub(crate) mod ffi {
         handle: &PropertyHandleOpaque,
         val: *mut c_void,
     ) {
-        handle.0.update(val);
-        handle.0.register_as_dependency_to_current_binding();
+        let handle = Pin::new_unchecked(&handle.0);
+        handle.update(val);
+        handle.register_as_dependency_to_current_binding();
     }
 
     /// Mark the fact that the property was changed and that its binding need to be removed, and
@@ -1661,7 +1755,7 @@ pub(crate) mod ffi {
 
     /// Set a binding
     ///
-    /// The current implementation will do usually two memory alocation:
+    /// The current implementation will do usually two memory allocation:
     ///  1. the allocation from the calling code to allocate user_data
     ///  2. the box allocation within this binding
     /// It might be possible to reduce that by passing something with a
@@ -1691,7 +1785,7 @@ pub(crate) mod ffi {
 
     /// Set a binding using an already allocated building holder
     ///
-    //// (take ownershipo of the binding)
+    //// (take ownership of the binding)
     #[no_mangle]
     pub unsafe extern "C" fn sixtyfps_property_set_binding_internal(
         handle: &PropertyHandleOpaque,
@@ -1709,7 +1803,7 @@ pub(crate) mod ffi {
     /// Destroy handle
     #[no_mangle]
     pub unsafe extern "C" fn sixtyfps_property_drop(handle: *mut PropertyHandleOpaque) {
-        core::ptr::read(handle);
+        core::ptr::drop_in_place(handle);
     }
 
     fn c_set_animated_value<T: InterpolatedPropertyValue + Clone>(
@@ -1992,6 +2086,6 @@ pub(crate) mod ffi {
     /// Destroy handle
     #[no_mangle]
     pub unsafe extern "C" fn sixtyfps_property_tracker_drop(handle: *mut PropertyTrackerOpaque) {
-        core::ptr::read(handle as *mut PropertyTracker);
+        core::ptr::drop_in_place(handle as *mut PropertyTracker);
     }
 }
