@@ -11,11 +11,10 @@ LICENSE END */
 //! Exposed Window API
 
 use crate::graphics::Point;
-use crate::input::{KeyEvent, MouseEventType, MouseInputState, TextCursorBlinker};
+use crate::input::{KeyEvent, MouseEvent, MouseInputState, TextCursorBlinker};
 use crate::items::{ItemRc, ItemRef, ItemWeak};
 use crate::properties::PropertyTracker;
 use crate::slice::Slice;
-use crate::ImageReference;
 use crate::{
     component::{ComponentRc, ComponentWeak},
     SharedString,
@@ -27,7 +26,7 @@ use std::rc::{Rc, Weak};
 
 /// This trait represents the interface that the generated code and the run-time
 /// require in order to implement functionality such as device-independent pixels,
-/// window resizing and other typicaly windowing system related tasks.
+/// window resizing and other typically windowing system related tasks.
 pub trait PlatformWindow {
     /// Registers the window with the windowing system.
     fn show(self: Rc<Self>);
@@ -65,14 +64,7 @@ pub trait PlatformWindow {
         item_graphics_cache: &crate::item_rendering::CachedRenderingData,
         unresolved_font_request_getter: &dyn Fn() -> crate::graphics::FontRequest,
         reference_text: Pin<&crate::properties::Property<SharedString>>,
-    ) -> Option<Box<dyn crate::graphics::FontMetrics>>;
-
-    /// Return the size of the image referenced by the specified resource, multiplied by the window
-    /// scale factor.
-    fn image_size(
-        &self,
-        source: Pin<&crate::properties::Property<ImageReference>>,
-    ) -> crate::graphics::Size;
+    ) -> Box<dyn crate::graphics::FontMetrics>;
 
     /// Return self as any so the backend can upcast
     fn as_any(&self) -> &dyn core::any::Any;
@@ -170,6 +162,9 @@ impl Window {
     /// Associates this window with the specified component. Further event handling and rendering, etc. will be
     /// done with that component.
     pub fn set_component(&self, component: &ComponentRc) {
+        if let Some(w) = self.platform_window.get() {
+            w.close_popup(); // ensure the popup is closed
+        }
         self.focus_item.replace(Default::default());
         self.mouse_input_state.replace(Default::default());
         self.component.replace(ComponentRc::downgrade(component));
@@ -196,12 +191,12 @@ impl Window {
     /// * `pos`: The position of the mouse event in window physical coordinates.
     /// * `what`: The type of mouse event.
     /// * `component`: The SixtyFPS compiled component that provides the tree of items.
-    pub fn process_mouse_input(self: Rc<Self>, pos: Point, what: MouseEventType) {
+    pub fn process_mouse_input(self: Rc<Self>, event: MouseEvent) {
         crate::animations::update_animations();
         let component = self.component.borrow().upgrade().unwrap();
         self.mouse_input_state.set(crate::input::process_mouse_input(
             component,
-            crate::input::MouseEvent { pos, what },
+            event,
             &ComponentWindow::new(self.clone()),
             self.mouse_input_state.take(),
         ));
